function [tarray zarray] = ODE1(eqn, tspan, z0, options)
%Solve any ODE with Eulers method
tstart = tspan(1); 
tend = tspan(2);

%unpack options
RelTol = options.RelTol;    % max allowable relative error
maxtime = options.maxtime;  % max computer time used

tic;    % start keeping track of time




% The number of points in progressively more accurate solutions
npoints_array = 5*2.^[1:30];  % 10, 20, 30, 80, 160 ... 5*2^30.
%Initialize array of solutions at the final time
zendarray     = zeros(30,length(z0)); % array of solutions at final time

% Outer loop, progressively more accurate solutions, 
% Each time through the loop is a new solution.
    for j = 1:30  % iteration loop
        npoints   = npoints_array(j); %number of points in this solution
        disp(['Number of points this iteration is: ' num2str(npoints)])
        % Initialize various arrays
        tarray      = linspace(tstart, tend, npoints); %Soln at these times
        zarray      = zeros(npoints, length(z0)); % Claim memory for zarray
        z           = z0;  % z is present value of z, used in loop below
        zarray(1,:) = z';     % The first row is the initial condition z0.

        %%%%%%%%%%%%%%%%%%
        for i = 1:npoints-1   % Euler's method is in this 6-line loop
            t             = tarray(i);
            zdot          = eqn(t, z);
            h             = tarray(i+1) - tarray(i);
            znew          = z + zdot*h; % The key calculation in this whole process 
            zarray(i+1,:) = z';         % add a new row to the solution array
            z             = znew;       % the new value of z becomes the present value
    
            if toc>maxtime              % Timeout if it is taking too long
                disp(['  *** Max time has passed: '  num2str(maxtime) ' seconds. ***'])
                disp(['      (Solution up to timeout has been saved.' ...
                      ' Increase maxtime or RelTol or both.)'])

                tarray = tarray(1:i);     % the time so far
                zarray = zarray(1:i,:);   % save the solution so far
                break  % breaks out of Euler method loop
                
            end % of max time  check if
        end  %  of Euler's method
        %%%%%%%%%%%%%%%%%%
        
        if toc>maxtime    % Timeout if it is taking too long
            break  % breaks out of iterative solution loop
        end % check max time if
        
        zendarray(j,:) = zarray(end,:);    % add to list of solutions the last line of most recent solution
    
        if j>1  % can only compare solutions if you have more than one solution
             % More or less arbitrary measure of error
             error = (zendarray(j,:) - zendarray(j-1,:)); % differences
             error = sum(abs(error)); % sum all the differences as a measure of error
             error = error/sum(zendarray(j,:)); %Compare error to value of z
             disp(['   Error this iteration is: '  num2str(error)])
             disp(['   Time spent solving so far is: '  num2str(toc)  ' seconds.'])
             if error <  RelTol 
                disp('Success! RelTol achieved within allowed time.')
                break;   % exits the outer loop
             end % of error if
             
        end % of error check if
        
        
        
    end % of iteration loop


end % of function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
