%Main
%1) Driver, 2) RHS  (Euler's method is separated
%  Andy Ruina,      Feb 19, 2021
clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set up, driver
% parameters, ICs, time span, call the solver
t_start = 0; t_end = 2*pi;  npoints = 201;
tspan= [t_start, t_end];
x0 = 1; v0 = 0;  z0 = [x0;v0];
p.k = 1;  p.m = 1; p.c = 3 ;

% Which ODE should be solved
myrhs  =  @(t,z)   harmonic_rhs(t,z,p);  % indirect way  of passing parameters
%myrhs =  @(t,z)   exponential_rhs(t,z,p);  % indirect way  of passing parameters

options.RelTol  = 1e-5;   % Maximum acceptable fractional error
options.maxtime = 6;    % Give up if accuracy not achieved in this many seconds.


[tarray zarray] = ODE1(myrhs, tspan, z0, options);
xarray  =   zarray(:,1); varray = zarray(:,2);
figure(1); plot(xarray,varray); axis equal; title('v vs x')
figure(2); plot(tarray, xarray); title('x vs t')
axis equal
shg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




% ODES ARE BELOW
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RHS function
% Spring and mass ODE in first order form
function  zdot =  harmonic_rhs(t, z, p)
    x= z(1); v= z(2);
    k = p.k; m= p.m;
    Force  = -k*x ;
xdot =     v;
vdot = Force/m;

zdot = [xdot;vdot];
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Another RHS function: exponential growth
% Our differential eqn in first order form
function  zdot =  exponential_rhs(t, z, p)
    x  = z(1); 
    c  = p.c;
xdot   = c*x;
zdot = xdot;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%